@@ -62,6 +62,36 @@
 #include <conio.h>
 #define _WIN32_WINNT 0x0500 
 #include <windows.h>
+
+static int
+fsync (int fd)
+{
+  HANDLE h = (HANDLE) _get_osfhandle (fd);
+  DWORD err;
+
+  if (h == INVALID_HANDLE_VALUE)
+    {
+      errno = EBADF;
+      return -1;
+    }
+
+  if (!FlushFileBuffers (h))
+    {
+      err = GetLastError ();
+      switch (err)
+        {
+           case ERROR_INVALID_HANDLE:
+             errno = EINVAL;
+             break;
+
+           default:
+             errno = EIO;
+        }
+      return -1;
+    }
+
+  return 0;
+}
 #endif
 
 /* This makes hash table safer when debugging */
@@ -119,8 +149,6 @@ static GConfDatabase*       obtain_database (GSList *addresses,
 static void                 drop_old_databases (void);
 static gboolean             no_databases_in_use (void);
 
-static void gconf_handle_segv (int signum);
-
 /*
  * Flag indicating that we are shutting down, so return errors
  * on any attempted operation. We do this instead of unregistering with
@@ -431,22 +459,6 @@ signal_handler (int signo)
   ++in_fatal;
   
   switch (signo) {
-    /* Fast cleanup only */
-  case SIGSEGV:
-#ifdef SIGBUS
-  case SIGBUS:
-#endif
-  case SIGILL:
-    enter_shutdown ();
-#ifndef G_OS_WIN32
-    if (g_getenv ("DISPLAY"))
-      gconf_handle_segv (signo);
-#else
-    gconf_handle_segv (signo);
-#endif
-    abort ();
-    break;
-
   case SIGFPE:
 #ifdef SIGPIPE
   case SIGPIPE:
@@ -655,7 +667,7 @@ system_bus_message_handler (DBusConnection *connection,
 
 	  gconf_log (GCL_DEBUG, "System defaults changed.  Notifying.");
 
-	  addresses.data = "xml:merged:/etc/gconf/gconf.xml.system";
+	  addresses.data = "xml:merged:" GCONF_ETCDIR "/gconf.xml.system";
 	  addresses.next = NULL;
 	  system_sources = gconf_sources_new_from_addresses (&addresses, NULL);
 
@@ -844,28 +856,16 @@ main(int argc, char** argv)
   act.sa_mask    = empty_mask;
   act.sa_flags   = 0;
   sigaction (SIGTERM,  &act, NULL);
-  sigaction (SIGILL,  &act, NULL);
-  sigaction (SIGBUS,  &act, NULL);
-  sigaction (SIGFPE,  &act, NULL);
   sigaction (SIGHUP,  &act, NULL);
-  sigaction (SIGSEGV, &act, NULL);
-  sigaction (SIGABRT, &act, NULL);
   sigaction (SIGUSR1,  &act, NULL);
 
   act.sa_handler = SIG_IGN;
   sigaction (SIGINT, &act, NULL);
 #else
   signal (SIGTERM, signal_handler);
-  signal (SIGILL,  signal_handler);
-#ifdef SIGBUS
-  signal (SIGBUS,  signal_handler);
-#endif
-  signal (SIGFPE,  signal_handler);
 #ifdef SIGHUP
   signal (SIGHUP,  signal_handler);
 #endif
-  signal (SIGSEGV, signal_handler);
-  signal (SIGABRT, signal_handler);
 #ifdef SIGUSR1
   signal (SIGUSR1,  signal_handler);
 #endif
@@ -2263,7 +2263,7 @@ read_line (FILE *f)
         }
 
       len = strlen (buf);
-      if (buf[len - 1] == '\n')
+      if (len > 0 && buf[len - 1] == '\n')
 	buf[--len] = '\0';
 
       if (retval == NULL)
@@ -2710,10 +2710,3 @@ client_count (void)
     return g_hash_table_size (client_table);
 }
 
-
-static void
-gconf_handle_segv (int signum)
-{
-  return;
-}
-
